import gtk
import picoLowLevel
import picoData
import numarray as na

def scale_set_default_values(scale):
    scale.set_update_policy(gtk.UPDATE_CONTINUOUS)
    scale.set_digits(1)
    scale.set_value_pos(gtk.POS_TOP)
    scale.set_draw_value(True)

def create_arrow_button(arrow_type, shadow_type):
    button = gtk.Button();
    arrow = gtk.Arrow(arrow_type, shadow_type);
    button.add(arrow)
    button.show()
    arrow.show()
    return button

class pico(gtk.Window, picoLowLevel.PicoControl):
    def __init__(self, picoName = 'cyl lens', parent=None):
    	#init pico control
	picoIP = picoData.getFieldValue(picoData.picoId, 'IP', 'picoId', picoName)
        index, = picoData.picoId.field('picoId').match(picoName)
        nAxis = len(index)
	self.driver = picoData.picoId.field('driver')[index]
	self.motor = picoData.picoId.field('motor')[index]
	self.steps = na.array([500]*nAxis)
	self.Relative = na.zeros(nAxis)
	print "picoName = ", picoName
        print "available axis: ", picoData.picoId.field('axis')[index]
	print "picoIP = ", picoIP
        
        #intialisation of the telnet connection with the picoMotor
        picoLowLevel.PicoControl.__init__(self, picoIP)
        
        ## print "self.tn.host = ", self.tn.host
        for i in range(nAxis):
            self.SetVelocity(self.driver[i], self.motor[i], 2000)

	# Create the toplevel window
        gtk.Window.__init__(self)	
        try:
            self.set_screen(parent.get_screen())
        except AttributeError:
            self.connect('destroy', lambda *w: gtk.main_quit())

	self.set_title(picoName)
# 	self.set_default_size(300, 200) 
        
        table = gtk.Table(5, nAxis*4+2, False)
        self.add(table)
        self.entry = []
        self.entrySteps = []
        for i in range(nAxis):
            self.entry.append(gtk.Entry())
            self.entrySteps.append(gtk.Entry())
            frame = gtk.Frame(picoData.picoId.field('axis')[index[i]])
            self.entry[i].set_size_request(60, 20)
            self.entry[i].set_text(str(self.Relative[i]))
            self.entry[i].set_editable(False)
            frame.add(self.entry[i])
            table.attach(frame,
                         # X direction #          # Y direction
                         2, 3,                      i*4, 1+i*4,
                         0,     0,
                         0,                         0);
 
            button = create_arrow_button(gtk.ARROW_LEFT, gtk.SHADOW_ETCHED_IN)
            button.connect("clicked", self.move_cb, "left", i)

            table.attach(button,
                         # X direction #          # Y direction
                         1, 2,                      1+i*4, 2+i*4,
                         gtk.EXPAND | gtk.FILL,     gtk.EXPAND | gtk.FILL,
                         0,                         0);
  
            button = gtk.Button('ZERO')
            button.connect("clicked", self.move_cb, "zero", i)
	
            table.attach(button,
                         # X direction #          # Y direction
                         2, 3,                      1+i*4, 2+i*4,
                         gtk.EXPAND | gtk.FILL,     gtk.EXPAND | gtk.FILL,
                         0,                         0);
  
            button = create_arrow_button(gtk.ARROW_RIGHT, gtk.SHADOW_ETCHED_OUT)	
            button.connect("clicked", self.move_cb, "right", i)

            table.attach(button,
                         # X direction #          # Y direction
                         3, 4,                      1+i*4, 2+i*4,
                         gtk.EXPAND | gtk.FILL,     gtk.EXPAND | gtk.FILL,
                         0,                         0);
            
            frame = gtk.Frame("STEP SIZE")
            
            self.entrySteps[i].set_size_request(60, 20)
            self.entrySteps[i].set_text(str(self.steps[i]))
            self.entrySteps[i].connect("activate", self.enter_callback, i)
            frame.add(self.entrySteps[i])
            table.attach(frame,
                         # X direction #          # Y direction
                         2, 3,                      2+i*4, 3+i*4,
                         0,     0,
                         0,                         0);

            separator = gtk.HSeparator()
            table.attach(separator,
                         # X direction #          # Y direction
                         1, 4,                      3+i*4, 4+i*4,
                         gtk.EXPAND | gtk.FILL,     0,
                         0,                         0);
        
	button = gtk.Button('close')
        button.connect('clicked', lambda button, w=self: w.destroy())
        table.attach(button,
            # X direction #          # Y direction
            1, 4,                      5+i*4, 6+i*4,
            gtk.EXPAND | gtk.FILL,     0,
            0,                         0);
        
	button.set_flags(gtk.CAN_DEFAULT)
        button.grab_default()	
	
        self.show_all()
	
    def move_cb(self, widget, status, ndx):
    	print "status = ", status
	if (status == "right"):
		self.MoveRelative(self.driver[ndx],self.motor[ndx],self.steps[ndx])
		self.Relative[ndx] += self.steps[ndx]
		self.entry[ndx].set_text(str(self.Relative[ndx]))
	if (status == "left"):
		self.MoveRelative(self.driver[ndx],self.motor[ndx],-self.steps[ndx])
		self.Relative[ndx] -= self.steps[ndx]
		self.entry[ndx].set_text(str(self.Relative[ndx]))
	if (status == "zero"):
		self.MoveRelative(self.driver[ndx],self.motor[ndx],-self.Relative[ndx])
		self.Relative[ndx] = 0
		self.entry[ndx].set_text(str(self.Relative[ndx]))
		
    def enter_callback(self, widget, ndx):
	entry_text = self.entrySteps[ndx].get_text()
	self.steps[ndx] = int(entry_text)
	print "New step size: %d\n" % self.steps[ndx]


    
    
def main(pico):	
    functionToCall = picoData.tokenDict[pico]
    functionToCall()

    gtk.main()
	
if __name__ == '__main__':
    main()
